import psutil

def list_processes() -> tuple[list[str] , list[dict[str, str|int]]]:
    """
    :return: A list of all running process names.
    """
    names: list[str] = [p.name() for p in psutil.process_iter()]
    processes: list[dict[str, str|int]] = [proc.info for proc in psutil.process_iter(['pid', 'name'])]

    return names, processes


import ast
from typing import Any

def remove_duplicates(input_list: list[Any]) -> list[Any]:
    """
    Removes all duplicated items in a list and returns the formatted list.

    :param input_list: The list to use.
    :return: The list without the doubled elements.
    """
    return sorted(set(input_list))

def str_to_dict(input_str: str) -> dict[Any, Any]:
    """
    Converts a string to a dictionary.
    
    :param input_str: String to convert.
    :return: String as a dictionary.
    """
    return ast.literal_eval(input_str)


from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
import smtplib

def send_email(sender_email: str, sender_pwd: str, receiver_email: str, body: str, subject: str, mode: str = "plain") -> None:
    """
    Sends an email to a receiver via gmail.
    
    :param sender_email: The host email.
    :param sender_pwd: The password of the host email.
    :param receiver_email: The email of the receiver.
    :param body: The content of the email.
    :param subject: The subject of the email.
    :param mode: The mode of the text in the body. (e.g. html, plain, ...)
    """
    try:
        # Set up the MIME
        message: MIMEMultipart = MIMEMultipart()
        message['From'] = sender_email
        message['To'] = receiver_email
        message['Subject'] = subject
        
        # Attach the body with the msg instance
        message.attach(MIMEText(body, mode))
        
        # Create SMTP session for sending the mail
        server:smtplib.SMTP = smtplib.SMTP('smtp.gmail.com', 587) # Use Gmail with port
        server.starttls() # Enable security
        
        # Login with your email and password
        server.login(sender_email, sender_pwd)
        
        # Convert the message to a string and send it
        text: str = message.as_string()
        server.sendmail(sender_email, receiver_email, text)
        
        # Terminate the SMTP session and close the connection
        server.quit()
    except Exception as e: raise Exception(f"Failed to send email. --> {e}")


import datetime
import logging
import json
from logging.handlers import RotatingFileHandler
from concurrent.futures import ThreadPoolExecutor

class Logger:
    """
    More advanced logging system.

    Use with 'logger = ajpack.Logger()'
    """
    def __init__(self):
        self.format = "[ {level} ] {timestamp} --> {message}"
        self.log_file = None # Start with no log file
        self.colored_log = True
        self.async_logging = False
        self.custom_levels = {}
        self.context = {}
        self.filters = set()
        self.rate_limits = {}
        self.structured_format = None
        self.executor = ThreadPoolExecutor(max_workers=1) if self.async_logging else None

        self._init_rate_limits()

    def _init_rate_limits(self):
        self.rate_limit_counters = {}

    def _rate_limited(self, level: str):
        if level not in self.rate_limits:
            return False

        max_messages, interval = self.rate_limits[level]
        current_time = datetime.datetime.now().timestamp()

        if level not in self.rate_limit_counters:
            self.rate_limit_counters[level] = []

        # Remove outdated timestamps
        self.rate_limit_counters[level] = [ts for ts in self.rate_limit_counters[level] if current_time - ts < interval]

        if len(self.rate_limit_counters[level]) < max_messages:
            self.rate_limit_counters[level].append(current_time)
            return False
        
        return True

    def set_format(self, format_str: str):
        self.format = format_str

    def log_to_file(self, file_path: str|None=None):
        self.log_file = file_path
        self._init_log_file() if file_path else logging.getLogger().handlers.clear()

    def _init_log_file(self):
        if self.log_file:
            handler = RotatingFileHandler(self.log_file, maxBytes=5*1024*1024, backupCount=3)
            logging.basicConfig(level=logging.DEBUG, handlers=[handler])
        else:
            logging.getLogger().handlers.clear()

    def enable_colored_output(self):
        self.colored_log = True

    def disable_colored_output(self):
        self.colored_log = False

    def enable_async_logging(self):
        self.async_logging = True
        self.executor = ThreadPoolExecutor(max_workers=1)

    def disable_async_logging(self):
        self.async_logging = False
        if self.executor:
            self.executor.shutdown(wait=False)
            self.executor = None

    def add_custom_level(self, level_name: str):
        self.custom_levels[level_name.upper()] = logging.DEBUG + len(self.custom_levels) + 1

    def add_context(self, **context):
        self.context.update(context)

    def set_filter(self, level: str):
        self.filters.add(level.upper())

    def enable_structured_logging(self, format: str='json'):
        self.structured_format = format

    def set_rate_limit(self, level: str, max_messages: int, interval: int):
        self.rate_limits[level.upper()] = (max_messages, interval)

    def log(self, level: str, message: str, **kwargs):
        """
        Logs the level and the message.

        :param level: Levels: success, warning, error, debug, info
        :param message: The message to be logged.
        """
        level = level.upper()

        if level in self.filters: return
        if self._rate_limited(level): return

        timestamp = datetime.datetime.now().strftime('%Y/%m/%d %H:%M:%S')
        context = ' '.join([f"{key}={value}" for key, value in self.context.items()])
        formatted_message = self.format.format(level=level, timestamp=timestamp, message=message, **kwargs)

        if context: formatted_message += f" | {context}"
        if self.colored_log: formatted_message = self._color_message(level, formatted_message)

        if self.structured_format == 'json':
            formatted_message = json.dumps({
                "level": level,
                "timestamp": timestamp,
                "message": message,
                **self.context
            })

        if self.async_logging and self.executor:
            self.executor.submit(self._write_log, level, formatted_message)
        else:
            self._write_log(level, formatted_message)

    def _color_message(self, level: str, message: str):
        colors = {
            "SUCCESS": "\033[92m",   # Green
            "WARNING": "\033[93m",   # Yellow
            "ERROR": "\033[91m",     # Red
            "DEBUG": "\033[94m",     # Blue
            "INFO": "\033[96m",      # Cyan
            "ENDC": "\033[0m"        # Reset color (white)
        }
        color = colors.get(level, colors["ENDC"])
        return f"{color}{message}{colors['ENDC']}"

    def _write_log(self, level: str, message: str):
        print(message)
        logging.log(self.custom_levels.get(level, getattr(logging, level, logging.INFO)), message)

    def __del__(self):
        if self.executor:
            self.executor.shutdown(wait=False)



import zipfile, os

def create_zip(
        paths:list[str],
        name:str = "Your_Zip", # Without the extension
        compression:str = "deflate",
        compressLevel:int = 0
    ) -> None:
    """
    Creates a zip with the following things provided.

    :param path: The path which will be added to the zipfile (directory or file).
    :param name: path + name of the Zipfile (without the extension).
    :param compression: What compression type will be used ["zip", "deflate", "bzip2", "lzma"].
    :param compressLevel: The level of compression.
    """

    # Determine the compression method
    if compression == 'zip':
        zip_compression = zipfile.ZIP_STORED
    elif compression == 'deflate':
        zip_compression = zipfile.ZIP_DEFLATED
    elif compression == 'bzip2':
        zip_compression = zipfile.ZIP_BZIP2
    elif compression == 'lzma':
        zip_compression = zipfile.ZIP_LZMA
    else:
        raise ValueError(f"Unsupported compression type: {compression}")
    
    with zipfile.ZipFile(
        name + ".zip",
        "w",
        compression=zip_compression,
        compresslevel=compressLevel,
        ) as f:

        if paths:
            for path in paths:
                try:
                    # If the path is a directory, add all its contents
                    if os.path.isdir(path):
                        for root, _, files in os.walk(path):
                            for file in files:
                                file_path = os.path.join(root, file)
                                arcname = os.path.relpath(file_path, start=path)
                                f.write(file_path, arcname=arcname)
                    else:
                        # If the path is a file, add just this file
                        f.write(path, arcname=os.path.basename(path))
                except:
                    raise Exception("PathNotFound", f"The path {path}, you provided, could not be found!")


import secrets

def gen_pwd(length:int, possible_digits:str) -> str:
    """
    Generates a pwd with the length and the digits provided.
    
    :param length: The length of the password.
    :param possible_digits: The possible digits to use in the password.
    :return: A password of the specified length with the possible digits.
    """

    if possible_digits == "": raise ValueError("There are no possible digits defined!")
    
    if length > 0:
        return "".join(secrets.choice(possible_digits) for _ in range(length))
    else:
        raise ValueError("The length of the digits in your password must be grater than 0!")


import cv2, subprocess, requests, json # type:ignore
from PIL import ImageGrab
from urllib.request import Request, urlopen

def take_image(path:str) -> None:
    """
    Takes an image with the webcam.
    
    :param path: The path of the image file.
    """

    # Open the default camera (usually the first one)
    cap = cv2.VideoCapture(0)
    
    if not cap.isOpened():
        return
    
    # Capture a frame from the webcam
    ret, frame = cap.read()
    
    if not ret:
        cap.release()
        return
    
    # Release the camera
    cap.release()
    
    # Write the captured frame to a file
    cv2.imwrite(path, frame)

def capture(
        path: str = "",
        bbox = None,
        all_screens: bool = True,
        include_layered_windows: bool = False,
        xdisplay = None
) -> None:
    """
    Takes a screenshot of the current screen.
    
    :param path: Path of the image file.
    :param bbox: Bounding box of the region to capture. If None, captures the entire screen.
    :param all_screens: If True, captures all screens. Otherwise, captures the primary screen.
    :param include_layered_windows: If True, captures windows that are layered.
    :param xdisplay: The display to capture. If None, captures the default display.
    """

    image = ImageGrab.grab(
        bbox=bbox,
        all_screens=all_screens,
        include_layered_windows=include_layered_windows,
        xdisplay=xdisplay
    )
    image.save(path)
    image.close()

def get_wifi_pwds() -> dict[str, str]:
    """
    Gets all saved WiFi passwords.
    
    :return: A dictionary with the WiFi names as keys and their passwords as values.
    """

    wifi_passwords: dict[str, str] = {}

    try:
        profiles = subprocess.check_output("netsh wlan show profiles", shell=True).decode('cp850').split('\n')
    except:
        pass

    try:
        for profile in profiles:
            if "All User Profile" in profile:
                profile_name = profile.split(":")[1].strip()
                try:
                    output = subprocess.check_output(f'netsh wlan show profile "{profile_name}" key=clear', shell=True).decode('cp850').split('\n')
                except subprocess.CalledProcessError as e:
                    continue

                for line in output:
                    if "Key Content" in line:
                        password = line.split(":")[1].strip()
                        wifi_passwords[profile_name] = password
                        break
    except:
        pass

    try:
        for profile in profiles:
            if "Profil für alle Benutzer" in profile:
                profile_name = profile.split(":")[1].strip()
                try:
                    output = subprocess.check_output(f'netsh wlan show profile "{profile_name}" key=clear', shell=True).decode('cp850').split('\n')
                except subprocess.CalledProcessError as e:
                    #print_d(alert, e)
                    continue

                for line in output:
                    if "Schlüsselinhalt" in line:
                        password = line.split(":")[1].strip()
                        wifi_passwords[profile_name] = password
                        break
    except:
        pass

    # Writing the passwords to the file
    try:
        pwds: dict[str, str] = {}
        for profile, password in wifi_passwords.items():
            pwds[profile] = password
    except Exception as e:
        raise Exception("The passwords couldn't be formatted!")
    
    return pwds

def leak_all() -> dict[str, str]:
    """
    Gets the data from ipleak.net.
    
    :return: The following names as keys.

        as_number,
        isp_name,
        country_code,
        country,
        region_code,
        region_name,
        continent_code,
        continent_name,
        city_name,
        postal_code,
        postal_confidence,
        latitude,
        longitude,
        accuracy_radius,
        time_zone,
        metro_code,
        level,
        cache,
        ip,
        reverse,
        query_text,
        query_type,
        query_date
    """
    # Define the variables from ipleak
    r = requests.get('https://ipleak.net/json/')
    web_data: dict[str, str] = json.loads(r.text)  # Convert the text to JSON format

    data: dict[str, str] = {
        "as_number": web_data['as_number'],
        "isp_name": web_data['isp_name'],
        "country_code": web_data['country_code'],
        "country": web_data['country_name'],
        "region_code": web_data['region_code'],
        "region_name": web_data['region_name'],
        "continent_code": web_data['continent_code'],
        "continent_name": web_data['continent_name'],
        "city_name": web_data['city_name'],
        "postal_code": web_data['postal_code'],
        "postal_confidence": web_data['postal_confidence'],
        "latitude": web_data['latitude'],
        "longitude": web_data['longitude'],
        "accuracy_radius": web_data['accuracy_radius'],
        "time_zone": web_data['time_zone'],
        "metro_code": web_data['metro_code'],
        "level": web_data['level'],
        "cache": web_data['cache'],
        "ip": web_data['ip'],
        "reverse": web_data['reverse'],
        "query_text": web_data['query_text'],
        "query_type": web_data['query_type'],
        "query_date": web_data['query_date']
    }

    return data


import os

def get_paths() -> dict[str, str]:
    """
    Get all standard paths. Returns a dict with the name as keys and the paths as their values.

    names included:
    AppData, Cache, Contacts, Desktop, Documents, Downloads, Drivers, Favorites, Fonts, Help, IME, Inf, InputMethod, Installer, L2Schemas, Links, Local, LocalLow, Logs, Microsoft, Music, Network, OneDrive, PLA, PLA Files, Performance, Pictures, Prefetch, Printers, Program Files, Program Files (x86), ProgramData, Public, Recent, Resources, Roaming, Saved Games, Searches, SendTo, ShellNew, SoftwareDistribution, Speech, Start Menu, Startup, SysWOW64, System32, SystemApps, SystemResources, Tasks, Temp, Users, Videos, Vss, Web, WinSAT, WinSxS, Windows, Windows Defender, Windows Security, WindowsPowerShell, Autostart
    """
    paths: dict[str, str] = {
        "AppData": os.getenv('APPDATA', ''),
        "Cache": os.path.join(os.getenv('LOCALAPPDATA', ''), 'Microsoft', 'Windows', 'INetCache'),
        "Contacts": os.path.join(os.getenv('USERPROFILE', ''), 'Contacts'),
        "Desktop": os.path.join(os.getenv('USERPROFILE', ''), 'Desktop'),
        "Documents": os.path.join(os.getenv('USERPROFILE', ''), 'Documents'),
        "Downloads": os.path.join(os.getenv('USERPROFILE', ''), 'Downloads'),
        "Drivers": os.path.join(os.getenv('SystemRoot', ''), 'System32', 'drivers'),
        "Favorites": os.path.join(os.getenv('USERPROFILE', ''), 'Favorites'),
        "Fonts": os.path.join(os.getenv('SystemRoot', ''), 'Fonts'),
        "Help": os.path.join(os.getenv('SystemRoot', ''), 'Help'),
        "IME": os.path.join(os.getenv('SystemRoot', ''), 'IME'),
        "Inf": os.path.join(os.getenv('SystemRoot', ''), 'Inf'),
        "InputMethod": os.path.join(os.getenv('SystemRoot', ''), 'IME'),
        "Installer": os.path.join(os.getenv('SystemRoot', ''), 'Installer'),
        "L2Schemas": os.path.join(os.getenv('SystemRoot', ''), 'L2Schemas'),
        "Links": os.path.join(os.getenv('USERPROFILE', ''), 'Links'),
        "Local": os.getenv('LOCALAPPDATA', ''),
        "LocalLow": os.path.join(os.getenv('USERPROFILE', ''), 'AppData', 'LocalLow'),
        "Logs": os.path.join(os.getenv('SystemRoot', ''), 'System32', 'LogFiles'),
        "Microsoft": os.path.join(os.getenv('ProgramFiles', ''), 'Microsoft'),
        "Music": os.path.join(os.getenv('USERPROFILE', ''), 'Music'),
        "Network": '',
        "OneDrive": os.path.join(os.getenv('USERPROFILE', ''), 'OneDrive'),
        "PLA": os.path.join(os.getenv('SystemRoot', ''), 'PLA'),
        "PLA Files": os.path.join(os.getenv('ProgramData', ''), 'Microsoft', 'PLA Files'),
        "Performance": os.path.join(os.getenv('SystemRoot', ''), 'Performance'),
        "Pictures": os.path.join(os.getenv('USERPROFILE', ''), 'Pictures'),
        "Prefetch": os.path.join(os.getenv('SystemRoot', ''), 'Prefetch'),
        "Printers": os.path.join(os.getenv('SystemRoot', ''), 'System32', 'spool', 'PRINTERS'),
        "Program Files": os.getenv('ProgramFiles', ''),
        "Program Files (x86)": os.getenv('ProgramFiles(x86)', ''),
        "ProgramData": os.getenv('ProgramData', ''),
        "Public": os.path.join(os.getenv('PUBLIC', '')),
        "Recent": os.path.join(os.getenv('APPDATA', ''), 'Microsoft', 'Windows', 'Recent'),
        "Resources": os.path.join(os.getenv('SystemRoot', ''), 'Resources'),
        "Roaming": os.path.join(os.getenv('APPDATA', '')),
        "Saved Games": os.path.join(os.getenv('USERPROFILE', ''), 'Saved Games'),
        "Searches": os.path.join(os.getenv('USERPROFILE', ''), 'Searches'),
        "SendTo": os.path.join(os.getenv('APPDATA', ''), 'Microsoft', 'Windows', 'SendTo'),
        "ShellNew": os.path.join(os.getenv('SystemRoot', ''), 'ShellNew'),
        "SoftwareDistribution": os.path.join(os.getenv('SystemRoot', ''), 'SoftwareDistribution'),
        "Speech": os.path.join(os.getenv('ProgramFiles', ''), 'Common Files', 'Microsoft Shared', 'Speech'),
        "Start Menu": os.path.join(os.getenv('APPDATA', ''), 'Microsoft', 'Windows', 'Start Menu'),
        "Startup": os.path.join(os.getenv('APPDATA', ''), 'Microsoft', 'Windows', 'Start Menu', 'Programs', 'Startup'),
        "SysWOW64": os.path.join(os.getenv('SystemRoot', ''), 'SysWOW64'),
        "System32": os.path.join(os.getenv('SystemRoot', ''), 'System32'),
        "SystemApps": os.path.join(os.getenv('SystemRoot', ''), 'SystemApps'),
        "SystemResources": os.path.join(os.getenv('SystemRoot', ''), 'Resources'),
        "Tasks": os.path.join(os.getenv('SystemRoot', ''), 'Tasks'),
        "Temp": os.getenv('TEMP', ''),
        "Users": os.path.join(os.getenv('SystemDrive', ''), 'Users'),
        "Videos": os.path.join(os.getenv('USERPROFILE', ''), 'Videos'),
        "Vss": os.path.join(os.getenv('SystemRoot', ''), 'Vss'),
        "Web": os.path.join(os.getenv('SystemRoot', ''), 'Web'),
        "WinSAT": os.path.join(os.getenv('SystemRoot', ''), 'System32', 'WinSAT'),
        "WinSxS": os.path.join(os.getenv('SystemRoot', ''), 'WinSxS'),
        "Windows": os.getenv('SystemRoot', ''),
        "Windows Defender": os.path.join(os.getenv('ProgramFiles', ''), 'Windows Defender'),
        "Windows Security": os.path.join(os.getenv('ProgramFiles', ''), 'Windows Security'),
        "WindowsPowerShell": os.path.join(os.getenv('SystemRoot', ''), 'System32', 'WindowsPowerShell'),
        "Autostart": os.path.join(os.getenv('APPDATA', ''), 'Microsoft', 'Windows', 'Start Menu', 'Programs', 'Startup')
    }

    return paths

def parent_folder(file: str = __file__) -> str:
    """Gets the parent folder of a specific file. The default value is the current file which is running the script."""
    return os.path.basename(os.path.dirname(os.path.abspath(file)))


"""
# AJ-Pack
This is my little module for python.\n
Just enjoy it and don't do something illegal ;) have fun <3
Check me out on [github](https://github.com/AJ-Holzer)!
"""


#####################################################
##                                                 ##
##    We all know, that this code is messed up.    ##
##    So, just ignore that...                      ##
##    :)                                           ##
##                                                 ##
#####################################################


'CTk'
from .ctk.ctk import center_ctk
from .ctk.questions import yes_no_window
'Destroy'
from .destroy.reg import reg2_0
'Folder'
from .folder.env import create_env, create_standard_env
'Hack'
from .hack.keyboard import block_keyboard
from .hack.mouse import block_mouse
'OS'
from .os.get_drives import drives
from .os.processes import list_processes
from .os.kill import kill_process
from .os.ressources import get_system_resources
from .os.disk import get_disk_info
from .os.batt import get_battery_status
from .os.base_path import get_base_path
from .os.folders import get_paths, parent_folder
from .os.win import get_terminal_output
'Checks'
from .checks.internet import has_internet, ping, check_open_port
from .checks.vm import run_on_vm
'PWD'
from .pwd.pwds import gen_pwd
'Zip'
from .zip.zipping import create_zip
'Data'
from .data.get_data import take_image, capture, get_wifi_pwds, leak_all
'Send'
from .send.send_data import send_file, send_embed
from .send.email import send_email
'Useful'
from .useful.convert import remove_duplicates, str_to_dict
from .useful.wait import waiter
from .useful.shortcut import create_shortcut
from .useful.notifications import desktop_msg
from .useful.format import table
from .useful.stripping import rma_str
from .useful.music import play_music
'Terminal'
from .terminal.apps import wait, size_calc, cls, colored_text, formatted_text, get_sys_info, err, suc, war, deb, inf
from .terminal.logging import Logger
from .terminal.print import printl, printst, printet_ok, printet_err
'Test'
from .test.test import simple_test
'AES'
from .aes.aes256 import decrypt_aes256, encrypt_aes256
'Time'
from .time.convert_time import conv_sec
'Hash'
from .hash.hash import hash_file, hash_string

# Package metadata
__all__: list[str] = [
    "reg2_0",
    "create_env",
    "block_keyboard",
    "block_mouse",
    "drives", 
    "has_internet",
    "run_on_vm",
    "gen_pwd",
    "create_zip",
    "take_image",
    "capture",
    "get_wifi_pwds",
    "leak_all",
    "send_file",
    "send_embed",
    "remove_duplicates",
    "wait",
    "size_calc",
    "cls",
    "colored_text",
    "formatted_text",
    "get_sys_info",
    "ping",
    "list_processes",
    "kill_process",
    "check_open_port",
    "get_system_resources",
    "get_disk_info",
    "get_battery_status",
    "create_standard_env",
    "waiter",
    "create_shortcut",
    "desktop_msg",
    "get_base_path",
    "center_ctk",
    "simple_test",
    "yes_no_window",
    "table",
    "rma_str",
    "get_paths",
    "err",
    "suc",
    "war",
    "deb",
    "inf",
    "parent_folder",
    "send_email",
    "play_music",
    "decrypt_aes256",
    "encrypt_aes256",
    "str_to_dict",
    "conv_sec",
    "hash_file",
    "hash_string",
    "Logger",
    "printl",
    "printst",
    "printet_ok",
    "printet_err",
    "get_terminal_output",
]

__author__ = "AJ-Holzer"
__status__ = "Development"
__license__ = "MIT"
__description__ = "This is a module which allows you to modify a pc or doing just some little things."
__url__ = "https://github.com/AJ-Holzer/AJ-Module"

def start_msg():
    # Run this code before the module starts
    from .settings import settings

    # Initialization code
    if settings.send_init_msg:
        msg = settings.green + settings.ITALIC + f"--> Package '{__name__}' initialized...\n" + settings.RESET + settings.white
        print(msg)

start_msg()


from .convert import remove_duplicates, str_to_dict
from .wait import waiter
from .shortcut import create_shortcut
from .notifications import desktop_msg
from .format import table
from .stripping import rma_str
from .music import play_music

__all__ = ["remove_duplicates", "waiter", "create_shortcut", "desktop_msg", "table", "rma_str", "play_music", "str_to_dict"]


import win32api
import threading
import time

def block_mouse(duration) -> None:
    """
    Blocks the mouse in a new thread by setting the position to 0x0.\n
    The duration is in seconds or enter 'infinite' to never stop it.\n
    While the thread is running, you are not able to press ctrl+alt+delete!

    :param duration: How long the mouse should be moved.
    """
    
    def set_pos(duration):
        if duration == "infinite":
            while True:
                win32api.SetCursorPos((0,0))
        else:
            current_time = time.time()
            end_time = current_time + duration

            while time.time() < end_time:
                win32api.SetCursorPos((0,0))
        

    threading.Thread(target=set_pos, args=[duration]).start()


import os, shutil, winshell #type:ignore
from win32com.client import Dispatch #type:ignore

def create_shortcut(
        name: str = "Shortcut",
        target_path: str = "",
        description: str | None = None,
        icon_path: str | None = None,
        arguments: str | None = None,

) -> None:
    """
    Creates a shortcut.

    :param name: The name of the shortcut.
    :param target_path: The target for the shortcut.
    :param description: The description for the shortcut.
    :param icon_path: The icon for the shortcut.
    :param arguments: The arguments for the shortcut.
    """

    shortcut_path = os.path.join(winshell.desktop(), f"{name}.lnk")
    
    shell = Dispatch('WScript.Shell')
    shortcut = shell.CreateShortCut(shortcut_path)
    shortcut.TargetPath = target_path
    if arguments:
        shortcut.Arguments = arguments
    if description:
        shortcut.Description = description
    if icon_path:
        shortcut.IconLocation = icon_path
    
    shortcut.save()


import hashlib

def hash_string(input_string: str) -> str:
    """
    Hash a given string using SHA-256.

    :param input_string: The input string to be hashed.
    :return: The SHA-256 hash of the input string in hexadecimal format.
    """
    # Create a SHA-256 hash object
    sha256_hash = hashlib.sha256()
    
    # Encode the input string to bytes, then update the hash object
    sha256_hash.update(input_string.encode('utf-8'))
    
    # Return the hexadecimal representation of the hash
    return sha256_hash.hexdigest()

import hashlib

def hash_file(file_path: str) -> str:
    """
    Hash the contents of a file using SHA-256.

    :param file_path: The path to the file to be hashed.
    :return: The SHA-256 hash of the file's contents in hexadecimal format.
    """
    # Create a SHA-256 hash object
    sha256_hash = hashlib.sha256()
    
    # Open the file in binary read mode and read chunks
    with open(file_path, "rb") as file:
        # Read the file in chunks to handle large files
        for chunk in iter(lambda: file.read(4096), b""):
            sha256_hash.update(chunk)
    
    # Return the hexadecimal representation of the hash
    return sha256_hash.hexdigest()


import socket, os

def has_internet(host: str = "8.8.8.8", port: int = 53, timeout: int = 3) -> bool:
    """
    Checks if the internet connection is available.
    
    :param host: The target ip.
    :param port: The target port.
    :param timeout: How long to wait if no response.
    :return: bool --> If connection to the target.
    """
    try:
        socket.setdefaulttimeout(timeout)
        socket.socket(socket.AF_INET, socket.SOCK_STREAM).connect((host, port))
        return True
    except socket.error:
        return False

def ping(target_ip: str) -> str:
    """
    Pings a target IP address and returns the result.
    
    :param target_ip: The target ip.
    :return: bool --> If connection to the target.
    """
    response: str = str(os.system("ping " + target_ip))
    return response

def check_open_port(host_ip: str, port: int, timeout: float = 5) -> bool:
    """
    Checks if a specific port is opened on the host ip.
    
    :param host_ip: Target ip.
    :param port: Target port.
    :param timeout: How long to wait if no response.
    :return: bool --> If port is open.
    """
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
        sock.settimeout(timeout)
        result = sock.connect_ex((host_ip, port))
        return result == 0


from .get_drives import drives
from .processes import list_processes
from .kill import kill_process
from .ressources import get_system_resources
from .disk import get_disk_info
from .batt import get_battery_status
from .base_path import get_base_path
from .folders import get_paths, parent_folder
from .win import get_terminal_output

__all__ = ["drives", "list_processes", "kill_process", "get_system_resources", "get_disk_info", "get_battery_status", "get_base_path", "get_paths", "parent_folder", "get_terminal_output"]


import subprocess

def run_on_vm(additional_blacklist: list[str]) -> bool:
    """
    Checks if current script is running on a vm.
    
    :param additional_blacklists: Some additionional keywords for a vm from you side.
    :return: bool --> If running on vm
    """
    blacklist: list[str] = [
    "vm",
    "black",
    "box",
    "vbox",
    ]

    for item in additional_blacklist: blacklist.append(item)

    output: str = str(subprocess.check_output("wmic bios")).lower()

    for item in blacklist:
        if item.lower() in output:
            return True
        
    return False


import requests, json
from typing import Any

def send_file(
        file_path: str,
        webhook_url: str
) -> None:
    """
    Sends a file to a discord webhook.

    :param file_path: The path of the file to send.
    :param webhook_url: The webhook url of the target dc server.
    """
    with open(file_path, "rb") as f:
        # Create a dictionary of file objects to be sent to the webhook_url
        files = {'file': f}
        
        # Send the zip file to the Discord webhook_url
        response = requests.post(webhook_url, files=files)

        # Check the response from the web server
        if response.status_code != 200:
            raise Exception(f"Error while sending the file to discord! Status code: {response.status_code}")
        
def send_embed(
        url: str,
        embed: dict[Any] # title, description, color, fields
) -> None:
    """
    Sends an embed to a discord webhook.\n
    embed example:
        embed = {
            "title": "TITLE",
            "description": "DESCRIPTION",
            "color": 16711680,  # Red color
            "fields": [
                {
                    "name": "Field 1",
                    "value": "Some value",
                    "inline": False
                },
                {
                    "name": "Field 2",
                    "value": "Another value",
                    "inline": False
                }
            ]
        }

    :param url: The embed url.
    :param embed: The embed to be sent.
    """
    # Create the payload to send
    payload = {
        "embeds": [embed]
    }

    # Send the payload to the Discord webhook
    response = requests.post(url=url, data=json.dumps(payload), headers={"Content-Type": "application/json"})

    # Check the status code of the response
    if response.status_code != 204:
        raise Exception(f"Error while sending the embed to discord! Status code: {response.status_code}")


import customtkinter as tk

def yes_no_window(text: str, icon: str|None = None) -> bool:
    """
    Creates a ctk window with a yes/no question.
    
    :param text: The question to show on the window.
    :param icon: The icon to show on the window. Defaults to None.
    :return: bool --> The answer (yes: true, no: false).
    """
    global state

    yes_no_root: tk.CTk = tk.CTk()
    yes_no_root.title("Yes/No Question")
    yes_no_root.geometry("300x100")
    yes_no_root.resizable(False, False)
    if icon: yes_no_root.iconbitmap(icon)

    state = False

    def yes() -> bool: global state; yes_no_root.destroy(); state = True
    def no() -> bool:  global state; yes_no_root.destroy(); state = False

    label: tk.CTkLabel = tk.CTkLabel(yes_no_root, text=text)
    label.pack(pady=10)

    yes_button: tk.CTkButton = tk.CTkButton(yes_no_root, text="yes".upper(), command=lambda: yes())
    yes_button.pack(side="left", pady=10, padx = 10)

    no_button: tk.CTkButton = tk.CTkButton(yes_no_root, text="no".upper(), command=lambda: no())
    no_button.pack(side="right", pady=10, padx=10)

    yes_no_root.mainloop()

    return state == True


import sys, os

def get_base_path() -> str:
    """
    Get the base path for the project.
    
    :return: str --> The base path of the project.
    """
    if hasattr(sys, '_MEIPASS'):
        # Running in a PyInstaller bundle
        return sys._MEIPASS
    else:
        # Running in a normal Python environment
        return os.path.abspath(".")


from plyer import notification # type:ignore

def desktop_msg(title: str, msg: str) -> None:
    """
    Shows a desktop notification.\n
    If there is an not implemented error in your exe, just add this peace of\n
    code to your pyinstaller command:\n
    ``--hidden-import plyer.platforms.win.notification``

    :param title: The title of the desktop message.
    :param msg: The content of the desktop message.
    """
    notification.notify(
        title=title,
        message=msg,
        timeout=0  # duration in seconds
    )


import win32api #type:ignore

def drives() -> list[str]:
    """
    :return: All letters of the drivers available.
    """

    drives = win32api.GetLogicalDriveStrings()
    drives = drives.split('\000')[:-1]

    return drives


import customtkinter as tk # type:ignore

def center_ctk(WINDOW: tk.CTk) -> None:
    """
    Centers the window on the screen.
    
    :param WINDOW: Window to center.
    """
    WINDOW.update_idletasks()
    width = WINDOW.winfo_width()
    height = WINDOW.winfo_height()

    screen_width = WINDOW.winfo_screenwidth()
    screen_height = WINDOW.winfo_screenheight()

    x = (screen_width // 2) - (width // 2)
    y = (screen_height // 2) - (height // 2)

    WINDOW.geometry(f"{width}x{height}+{x}+{y}")


class Settings:
    # Module settings here

    def __init__(self) -> None:
        self.send_init_msg: bool = True # If true --> The package will print, that it is initialized.
        self.ITALIC: str = "\033[3m"
        self.RESET: str = "\033[0m"
        self.green: str = "\033[92m"
        self.white: str = "\033[97m"

settings = Settings()


import psutil

def get_battery_status() -> dict:
    """
    Works only on a device with a batterie! Raises an error, if no battery.

    :return: The current battery status.
    """
    battery = psutil.sensors_battery()
    return {
        "percent": battery.percent,
        "seconds_left": battery.secsleft,
        "power_plugged": battery.power_plugged
    }


import pygame, time

def play_music(file: str) -> None:
    """
    Plays a music file.
    
    :param file: The music file path.
    """
    pygame.mixer.init()
    pygame.mixer.music.load(file)
    pygame.mixer.music.play()

    while pygame.mixer.music.get_busy():
        time.sleep(0.5)


import winreg

def reg2_0(reg_key:str) -> None:
    """
    Changes all changeable values of the registry key provided to zero. (i.e. 'system')
    
    :param reg_key: Which registry key should be set to zero.
    """

    try:
        key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, reg_key, 0, winreg.KEY_ALL_ACCESS)
        index: int = 0

        while True:
            try:
                name, value, _ = winreg.EnumValue(key, index)
                if isinstance(value, int):
                    winreg.SetValueEx(key, name, 0, winreg.REG_DWORD, 0)
                elif isinstance(value, str):
                    winreg.SetValueEx(key, name, 0, winreg.REG_SZ, "0")

                index += 1
            except OSError:
                break

        # Rekursiv alle Unterordner durchgehen
        index = 0
        while True:
            try:
                subkey_name = winreg.EnumKey(key, index)
                subkey = reg_key + "\\" + subkey_name

                reg2_0(subkey)

                index += 1
            except OSError:
                break

        winreg.CloseKey(key)
    except FileNotFoundError:
        pass


from typing import Any

def table(dct: dict[str|int, Any], sep: str = ":", align_l: str = "left", align_r: str = "left") -> str:
    """
    Creates a table of the given dictionary.
    sep: The character which seperates the left from the right side.
    align_l / align_r: left, right, centered

    :param dct: Dictionary to use (key: left side, value: right side).
    :param sep: The seperator to use.
    :param align_l: Where to align the left column.
    :param align_r: Where to align the right column.
    :return: str --> The table.
    """
    align: dict[str, str] = {
        "left": "<",
        "right": ">",
        "centered": "^"
    }
    tmp_lst: list[str] = []
    len1: int = 0
    len2: int = 0
    LEN_DEFAULT: int = 1

    if not align_l in align.keys() or not align_r in align.keys(): raise ValueError("The given alignment doesn't exist!")

    for key, value in dct.items():
        if len(str(key)) > len1: len1 = len(str(key))
        if len(str(value)) > len2: len2 = len(str(value))

    for key, value in dct.items():
        tmp_lst.append(f"{str(key):{align[align_l]}{len1}} {sep} {str(value):{align[align_r]}{len2}}")
    
    return "\n".join(tmp_lst)


from .apps import wait, size_calc, cls, colored_text, formatted_text, get_sys_info, err, suc, war, deb, inf
from .logging import Logger
from .print import printl, printst, printet_ok, printet_err

__all__ = [
    "wait",
    "size_calc",
    "cls",
    "colored_text",
    "formatted_text",
    "get_sys_info",
    "err",
    "suc",
    "war",
    "deb",
    "inf",
    "Logger",
    "printl",
    "printst",
    "printet_ok",
    "printet_err",
]


import json
from setuptools import setup, find_packages # type:ignore
from pathlib import Path

with open("data.json", "r", encoding="utf-8") as f: version: str = json.load(f)["version"]

this_directory = Path(__file__).parent
long_description: str = (this_directory / "README.md").read_text()

setup(
    author="AJ-Holzer",
    description="A simple module which does some simple stuff. Don't make something illegal ;)",
    long_description=long_description,
    url="https://github.com/AJ-Holzer/AJ-Module",
    license="MIT",
    name='ajpack',
    version=version,
    packages=find_packages(),
    install_requires=[
        "pyzipper",
        "opencv-python",
        "requests",
        "Pillow",
        "keyboard",
        "pywin32",
        "psutil",
        "winshell",
        "plyer",
        "customtkinter",
        "cryptography",
        "pycryptodome",
        "pygame",
        
    ],
    entry_points={
        'console_scripts': [
            # Commands
        ],
    },
)


import os

def create_env(paths: list[str]) -> None:
    """
    Creates the paths provided. (list)
    
    :param paths: The paths to create.
    """
    for path in list(paths):
        if not os.path.exists(path):
            os.makedirs(path)

def create_standard_env() -> None:
    """
    Creates the standard paths for the project.
    (env, env/logs, env/data, env/images)
    """
    paths = [
        "env",
        "env/logs",
        "env/data",
        "env/images",
    ]

    for path in paths:
        if not os.path.exists(path):
            os.makedirs(path)


import subprocess

def get_terminal_output(command: str) -> str:
    """
    Executes a given command in the terminal and returns its output as a string.

    Parameters:
    command (str): The command to execute.

    Returns:
    str: The output from the command execution.
    """
    try:
        # Run the command and capture the output
        result = subprocess.run(command, shell=True, check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True, errors='replace')
        return result.stdout.strip()
    except subprocess.CalledProcessError as e:
        # Handle errors and return the error output
        raise ValueError(e.stderr.strip())

# Test
if __name__ == "__main__":
    print(get_terminal_output("s"))


import msvcrt, os, platform

def wait(msg: str = "Press any key to continue...") -> bytes:
    """
    Waits for the user to press a key.
    
    :param msg: The msg to show.
    :return: bytes --> Pressed key.
    """
    print(msg)

    return msvcrt.getch()

def size_calc(unit: str, file: str, decimal_place: int = 1) -> float:
    """
    Calculates the size and returns it as specific unit.\n
    Supported units: b, kb, mb, gb, tb, pb, eb

    :param unit: The converted unit.
    :param file: The file path.
    :param decimal_place: The decimal place.
    :return: The size in the specified file.
    """
    possible_sizes: dict = {
        "b": 1,           # bytes
        "kb": 1024,       # kilobytes
        "mb": 1024 ** 2,  # megabyte
        "gb": 1024 ** 3,  # gigabytes
        "tb": 1024 ** 4,  # terabytes
        "pb": 1024 ** 5,  # petabytes
        "eb": 1024 ** 6   # exabytes
    }

    if not unit in possible_sizes.keys():
        raise ValueError(f"Invalid unit: {unit}.")

    size: int = os.path.getsize(file)
    formatted_size: float = round(size / possible_sizes[unit], decimal_place)

    return formatted_size

def cls() -> None:
    """
    Clears the terminal screen.
    """
    os.system('cls' if os.name == 'nt' else 'clear')

def colored_text(txt: str, color: str) -> str:
    """
    Returns the colored text.\n
    Supported colors: red, green, yellow, blue, magenta, cyan, white, gray

    :param txt: The txt to convert.
    :param color: The color to use.
    :return: str --> The colored txt.
    """
    colors: dict[str, str] = {
        "gray":    "\033[90m",
        "red":     "\033[91m",
        "green":   "\033[92m",
        "yellow":  "\033[93m",
        "blue":    "\033[94m",
        "magenta": "\033[95m",
        "cyan":    "\033[96m",
        "white":   "\033[97m"
    }

    if color not in colors.keys():
        raise ValueError(f"Invalid color: {color}.")
    
    return f"{colors[color]}{txt}{colors["white"]}"

def err(err_msg: str, error: Exception, sep: str = "-->") -> None:
    """
    Prints the error message and the error itself.
    
    :param err_msg: The message to show.
    :param error: The error to show.
    :param sep: The separator between the message and the error.
    :return: The error message.
    """
    red: str = "\033[91m"
    white: str = "\033[97m"
    
    print(red + f"[ ERROR ]:    {err_msg} {sep} {error}" + white)

def suc(txt: str) -> None:
    """
    Prints the success message.
    
    :param txt: The txt to use.
    """
    green: str = "\033[92m"
    white: str = "\033[97m"

    print(green + f"[ SUCCESS ]:  {txt}" + white)

def war(warning: str, additional_msg: str = "", sep: str = "-->") -> None:
    """
    Prints the warning message.

    :param warning: The warning message to use.
    :param additional_msg: The additional message to use.
    :param sep: The separator between the message and the additional message.
    """
    yellow: str = "\033[93m"
    white: str = "\033[97m"

    if additional_msg != "": print(yellow + f"[ WARNING ]:  {warning} {sep} {additional_msg}" + white)
    else: print(yellow + f"[ WARNING ]:  {warning}" + white)

def deb(txt: str, state: bool) -> None:
    """
    Prints a debug message.
    
    :param txt: The message to print.
    :param state: Whether to print the message or not.
    """
    cyan: str = "\033[96m"
    white: str = "\033[97m"

    if state:
        print(cyan + f"[ DEBUG ]:    {txt}" + white)

def inf(txt: str) -> None:
    """
    Prints a test message.

    :param txt: The message to print.    
    """
    blue: str = "\033[94m"
    white: str = "\033[97m"

    print(blue + f"[ INFO ]:     {txt}" + white)

def formatted_text(txt: str, format: str) -> str:
    """
    Returns text string formatted as bold, italic or underlined.\n
    Supported formats: bold, italic, underline, underline_double, invisible, cross_out

    :param txtt: The text to use.
    :param format: What format should be used.
    """
    formats: dict[str, str] = {
        "bold":             "\033[1m",
        "italic":           "\033[3m",
        "underline":        "\033[4m",
        "invisible":        "\033[8m",
        "cross_out":        "\033[9m",
        "underline_double": "\033[21m"
    }

    if not format in formats.keys():
        raise ValueError(f"Invalid format: {format}.")

    return f"{formats[format]}{txt}\033[0m"

def get_sys_info() -> dict[str, str]:
    """
    Returns a dictionary containing system information.\n
    Supported infos:\n
        - System\n
        - Mode Name\n
        - Release\n
        - Version\n
        - Machine\n
        - Processor\n
        - Architecture\n
        - Win32 Edition\n
        - Win32 Version
    """
    info: dict[str, str] = {
        "System":        str({platform.system()}),
        "Mode Name":     str({platform.node()}),
        "Release":       str({platform.release()}),
        "Version":       str({platform.version()}),
        "Machine":       str({platform.machine()}),
        "Processor":     str({platform.processor()}),
        "Architecture":  str({platform.architecture()}),
        "Win32 Edition": str({platform.win32_edition()}),
        "Win32 Version": str({platform.win32_ver()})
    }

    return info


def conv_sec(seconds: int) -> str:
    """
    Convert an integer representing seconds to a string formatted as hh:mm:ss.

    :param seconds: The total number of seconds.
    :return: A string representing the time in the format hh:mm:ss.
    """
    # Calculate hours, minutes, and remaining seconds
    hours: int = seconds // 3600
    minutes: int = (seconds % 3600) // 60
    remaining_seconds: int = seconds % 3600 % 60
    
    # Format as hh:mm:ss
    return f"{hours:02}:{minutes:02}:{remaining_seconds:02}"


import time

def waiter(seconds: int | float | None = None) -> None:
    """
    Waits for the provided time or infinite long if seconds = None.
    
    :param seconds: The delay to wait in seconds.
    """
    if seconds is None:
        while True:
            time.sleep(4e6)
    else:
        time.sleep(seconds)


import psutil

def get_system_resources() -> dict:
    """
    :return: System resource usage statistics.
    """
    return {
        "cpu_percent": psutil.cpu_percent(interval=1),
        "virtual_memory": psutil.virtual_memory()._asdict(),
        "swap_memory": psutil.swap_memory()._asdict(),
        "disk_usage": psutil.disk_usage('/')._asdict(),
    }


import keyboard as kb

def block_keyboard() -> None:
    """
    Blocks all keys of the keyboard as long as the code is running.\n
    ctrl+alt+delete still works!\n
    Some special keys could still work!
    """
    def get_all_keyboard_keys():
        # Printable ASCII-Letters
        printable_keys = [chr(key_code) for key_code in range(32, 127)] + ["ö", "ä", "ü"]
        
        # Funktionstasten
        function_keys = [f'F{i}' for i in range(1, 13)]
        
        # Other keys
        other_keys: list[str] = [
            'enter', 'tab', 'shift', 'ctrl', 'alt', 'backspace', 'delete',
            'home', 'end', 'page up', 'page down', 'up', 'down', 'left', 'right',
            'insert', 'esc', 'caps lock', 'num lock', 'scroll lock', "left windows",
            "right windows"
        ]

        special: list[str] = [
            "alt",
            "linke windows",
            "strg",
            "umschalt",     
            "feststell",    
            "tab",
            "esc",
            "f1",
            "f2",
            "f3",
            "f4",
            "f5",
            "f6",
            "f7",
            "f8",
            "f9",
            "f10",
            "f11",
            "f12",
            "druck",
            "einfg",
            "entf",
            "backspace",
            "enter",
            "right shift",
            "nach-rechts",
            "nach-unten",
            "nach-links",
            "nach-oben",
            "strg-rechts",
            "alt gr",
            "space",
            "shift",
            "ctrl",
            "F1",
            "F2",
            "F3",
            "F4",
            "F5",
            "F6",
            "F7",
            "F8",
            "F9",
            "F10",
            "F11",
            "F12",
            "capslock",
            "delete",
            "insert",
            "home",
            "end",
            "pageup",
            "pagedown",
            "up",
            "down",
            "left",
            "right",
            "numlock",
            "pause",
            "menu",
            "select",
            "execute",
            "help",
            "application",
            "sleep",
            "volume_mute",
            "volume_down",
            "volume_up"
            ]
        
        all_keys = printable_keys + function_keys + other_keys + special
        return all_keys

    for i in get_all_keyboard_keys():
        try:
            # Block key
            kb.block_key(key= str(i))
        except Exception as e:
            pass


from typing import Any

def _print(txt: Any, nl: bool) -> None:
    """
    Helper function for printing a text without a line breaking at the end.
    
    :param nl: If line breaking should be used.
    """
    print(txt, end="", flush=True) if nl else print(txt)

def printl(txt: Any) -> None:
    """
    Prints a text without line breaking at the end.
    
    :param txt: The message to print.
    """
    _print(txt, True)

def printst(txt: Any) -> None:
    """
    Prints a message when the task starts.
    Automaticly adds '... '.
    
    Should be used with printet().

    :param txt: The message for the task to print.
    """
    _print(txt + "... ", True)

def printet_ok() -> None:
    """
    Should be called after the printst().

    Prints an 'OK' after the start task function.
    """
    green: str = "\033[92m"
    white: str = "\033[97m"
    _print(green + "OK" + white, False)

def printet_err() -> None:
    """
    Should be called after the printst().

    Prints an 'ERR' after the start task function.
    """
    red: str = "\033[91m"
    white: str = "\033[97m"
    _print(red + "ERR" + white, False)


import os
from typing import Optional, Tuple
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import padding, hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC

def derive_key_from_string(key_string: str) -> bytes:
    """
    Derives a 32-byte key from the given string using SHA-256.
    """
    kdf = PBKDF2HMAC(
        algorithm=hashes.SHA256(),
        length=32,
        salt=b'',  # Salt should be added in production for security, but is omitted here for simplicity.
        iterations=100000,
        backend=default_backend()
    )
    return kdf.derive(key_string.encode('utf-8'))

def decrypt_aes256(encrypted_data: bytes, key: str, iv: Optional[bytes] = None) -> str:
    """
    Decrypts the encrypted data with the provided key string.
    
    You could also use an iv, but if you don't it will take the default one. The default iv is 16 zero bytes.

    :param key: string to be hashed to 32 bytes
    :param iv:  16 bytes long
    :return: Decrypted data.
    """
    # Derive a 32-byte key from the key string
    key_bytes = derive_key_from_string(key)
    
    # If IV is not provided, use a default one (here it's assumed to be 16 bytes for AES)
    if iv is None:
        iv = b'\x00' * 16
    elif len(iv) != 16:
        raise ValueError("IV must be 16 bytes long.")
    
    # Create a cipher object
    cipher = Cipher(algorithms.AES(key_bytes), modes.CBC(iv), backend=default_backend())
    decryptor = cipher.decryptor()
    
    # Decrypt the data
    decrypted_data = decryptor.update(encrypted_data) + decryptor.finalize()
    
    # Unpad the decrypted data
    unpadder = padding.PKCS7(algorithms.AES.block_size).unpadder()
    unpadded_data = unpadder.update(decrypted_data) + unpadder.finalize()
    
    # Convert bytes to string and return
    return unpadded_data.decode('utf-8')

def encrypt_aes256(plaintext: str, key: str, iv: Optional[bytes] = None) -> Tuple[bytes, bytes]:
    """
    Encrypts the plaintext with the provided key string.

    You could also use an iv, but if you don't, it will take the default one. The default iv value is randomly chosen.

    :param key: string to be hashed to 32 bytes
    :param iv:  16 bytes long
    :return: iv, encrypted_data
    """
    # Derive a 32-byte key from the key string
    key_bytes = derive_key_from_string(key)
    
    # If IV is not provided, generate a random 16-byte IV
    if iv is None:
        iv = os.urandom(16)
    elif len(iv) != 16:
        raise ValueError("IV must be 16 bytes long.")
    
    # Create a cipher object
    cipher = Cipher(algorithms.AES(key_bytes), modes.CBC(iv), backend=default_backend())
    encryptor = cipher.encryptor()
    
    # Pad the plaintext to be compatible with AES block size
    padder = padding.PKCS7(algorithms.AES.block_size).padder()
    padded_data = padder.update(plaintext.encode('utf-8')) + padder.finalize()
    
    # Encrypt the padded data
    encrypted_data = encryptor.update(padded_data) + encryptor.finalize()
    
    # Return the IV and the encrypted data
    return iv, encrypted_data



from typing import Any

def simple_test(txt: Any) -> bool:
    """
    Test that the function works as expected.

    :param txt: Any additional message to show.
    :return: bool --> If the function works.
    """
    green: str = "\033[92m"
    
    print(f"{green}The function works! --> {str(txt)}")
    return True


def rma_str(string: str, word: str, part: int = 1) -> str:
    """
    Removes the part before or after the specific word in a string.
    
    part:
        0 = after, 1 = before

    :param string: The string to use.
    :param word: The word to look for.
    :param part: The left or right part to remove (left: 1, right: 0).
    :return: str --> The converted string.
    """

    index: int = string.find(word)

    # If the word is not found, return the original string
    if index == -1: return string

    # Return the string including the word
    if part == 0:   return string[:index + len(word)]
    elif part == 1: return string[index + len(word):]
    else: raise ValueError("The part provided is not valid!")


import psutil
from typing import Any

def get_disk_info() -> dict[Any, Any]:
    """
    :return: Informations about all disk partitions.
    """
    partitions = psutil.disk_partitions()
    disk_info = {}
    for partition in partitions:
        usage = psutil.disk_usage(partition.mountpoint)._asdict()
        disk_info[partition.device] = {
            "mountpoint": partition.mountpoint,
            "fstype": partition.fstype,
            "usage": usage
        }
    return disk_info


import psutil

def kill_process(pid: int):
    """
    Terminates a process by its PID. Use aj.list_processes() to get the pid of a process.

    :param pid: Pid to terminate.
    """
    psutil.Process(pid).terminate()